classes = ["bike", "bus", "car", "motor", "person", "rider", "traffic light", "traffic sign", "train", "truck"]

import pycocotools.coco, pycocotools.cocoeval
def object_detection_evaluate(gts, preds):
    '''
    object detection performance evaluation on customized dataset
    gts: list of ground truth object boundingboxes, in form of 
         [[[class_1, x1, y1, x2, y2], # the first object in the first image
           [class_2, x1, y1, x2, y2], # the second object in the first image
           ...
          ],
          [[class_1, x1, y1, x2, y2], # the first object in the second image
           [class_2, x1, y1, x2, y2], # the second object in the second image
           ...
          ]
         ]
    preds: list of predicted object boundingboxes, in form of
        [[[class_1, score, x1, y1, x2, y2], # the first object in the first image
           [class_2, score, x1, y1, x2, y2], # the second object in the first image
           ...
          ],
          [[class_1, score, x1, y1, x2, y2], # the first object in the second image
           [class_2, score, x1, y1, x2, y2], # the second object in the second image
           ...
          ]
         ]
    '''
    coco_gt = pycocotools.coco.COCO()
    coco_gt.dataset = {}
    coco_gt.dataset["images"] = []
    coco_gt.dataset["annotations"] = []
    k = 0
    for i, gt in enumerate(gts):
        for j in range(gt.shape[0]):
            coco_gt.dataset["images"].append({"id": i})
            coco_gt.dataset["annotations"].append({"image_id": i, "category_id": gt[j, 0],
                                                   "bbox": np.hstack([gt[j, 1:3], gt[j, 3:5] - gt[j, 1:3]]),
                                                   "area": np.prod(gt[j, 3:5] - gt[j, 1:3]),
                                                   "id": k, "iscrowd": 0})
            k += 1
            pass
        pass
    coco_gt.dataset["categories"] = [{"id": i, "supercategory": c, "name": c} for i, c in enumerate(classes)]
    coco_gt.createIndex()

    coco_pred = pycocotools.coco.COCO()
    coco_pred.dataset = {}
    coco_pred.dataset["images"] = []
    coco_pred.dataset["annotations"] = []
    k = 0
    for i, pred in enumerate(preds):
        for j in range(pred.shape[0]):
            coco_pred.dataset["images"].append({"id": i})
            coco_pred.dataset["annotations"].append({"image_id": i, "category_id": np.int(pred[j, 5]),
                                                     "score": pred[j, 4], "bbox": np.hstack([pred[j, 0:2], pred[j, 2:4] - pred[j, 0:2]]),
                                                     "area": np.prod(pred[j, 2:4] - pred[j, 0:2]),
                                                     "id": k, "iscrowd": 0})
            k += 1
            pass
        pass
    coco_pred.dataset["categories"] = [{"id": i, "supercategory": c, "name": c} for i, c in enumerate(classes)]
    coco_pred.createIndex()

    coco_eval = pycocotools.cocoeval.COCOeval(coco_gt, coco_pred, "bbox")
    coco_eval.params.imgIds = coco_gt.getImgIds()
    coco_eval.evaluate()
    coco_eval.accumulate()
    coco_eval.summarize()
    return coco_eval.stats


